.TH authuser 3
.SH NAME
authuser \- remote authentication library using the Authentication Server
.SH SYNTAX
.B #include <authuser.h>
.PP
.B int auth_casecmp(u,v);
.br
.B char *u;
.br
.B char *v;
.PP
.B char *auth_xline(user,fd,&in);
.PP
.B int auth_fd(fd,&in,&local,&remote);
.PP
.B char *auth_tcpuser(in,local,remote);
.PP
.B char *user;
.br
.B int fd;
.br
.B unsigned long in;
.br
.B unsigned short local;
.br
.B unsigned short remote;
.SH DESCRIPTION
The
.I authuser
library provides a simple interface for
finding out the remote identity
of a connection through the
Authentication Server
as specified by RFC 931.
.PP
Static strings
.B authuserauthor[],
.B authuserversion[],
.B authusercopyright[],
.B authuserwarranty[],
and
.B authuserhelp[]
contain the authorship notice,
version number,
copyright notice,
warranty information,
and help notice respectively.
.PP
.B auth_casecmp(u,v)
returns 0 if the strings are equal, 1 if the first is larger,
-1 if the second is larger. Case is ignored.
.PP
.B auth_xline(user,fd,&in)
returns a line of the form X-Auth-User: username or X-Forgery-By: username,
depending upon what the host on the other side of
.B fd
thinks of the user.
This is particularly appropriate for
mail and news headers.
The line is stored in a static area
which is overwritten on each call to
.B auth_xline.
If 
.B fd
is not a TCP connection
or authentication is impossible,
.B auth_xline
returns NULL, setting errno appropriately.
If user is NULL,
it never returns X-Forgery-By.
The line is not cr-lf terminated.
.B auth_xline
places the Internet address of the other host into in.
.PP
.B auth_fd(fd,&in,&local,&remote)
retrieves address information from the connection in socket
.B fd.
It places the
Internet address of the host on other side into
.B in
and the local and remote
TCP ports into 
.B local
and 
.B remote.
.B auth_fd
returns -1 upon error, setting errno appropriately.
.PP
.B auth_tcpuser(in,local,remote)
returns the name of the user on the other end of the TCP connection
between
.B remote@in
and
.B local.
If authentication is impossible,
.B auth_tcpuser
returns
NULL, setting errno appropriately.
The user name is stored in a static area
which is overwritten on each call to
.B auth_tcpuser
and
.B auth_xline.
.PP
.SH MACHINES
.I authuser
has been tested
on an Astronautics ZS-2
running ZSUnix,
a Sun 3 running SunOS, a Sun 4 running SunOS,
a Convex C-210 running Convex UNIX,
and several other machines.
.SH RESTRICTIONS
.I authuser
does no backslash interpretation
upon the remote user name.
Hopefully the next revision of RFC 931
will make clear exactly what backslash
interpretation should be going on.
.PP
.I authuser
does not use the operating system type
information provided by the Authentication Server.
.SH VERSION
authuser version 2.0, dated April 2, 1990.
.SH AUTHOR
Copyright 1990, Daniel J. Bernstein.
.SH REFERENCES
The authentication server is more secure than passwords
in some ways, but less secure than passwords in many ways.
(It's certainly better than no password at all---e.g., for
mail or news.)
It is not the final solution.
For an excellent discussion of security problems within
the TCP/IP protocol suite, see
Steve Bellovin's article
``Security Problems in the TCP/IP Protocol Suite.''
.SH "SEE ALSO"
authtcp(1),
authd(1),
attachport(1),
tcp(4)
