/* Copyright (c) 1990 Theo de Raadt deraadt@cpsc.ucalgary.ca
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by Theo de Raadt.
 */
%{
#ifndef lint
char Copyright[] =
"@(#) Copyright (c) 1990 Theo de Raadt.\nAll rights reserved.\n";
#endif not lint

#ifndef lint
static	char Sccsid[] = "@(#)perms.y 1.0 90/11/01 TDR";
#endif not lint

#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <netinet/in.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>
#include <rpc/types.h>
#include <rpc/rpc.h>
#include <rpcsvc/ypclnt.h>
#include <rpcsvc/yp_prot.h>
#include <syslog.h>
#include "y.tab.h"

#undef DEBUG

/* define SECURE if you want logins disallowed if errors show up
 * during a map lookup. If this is not set, YP failures, incorrect
 * maps, automount failure, etc.,will let anymore login until the
 * problem is fixed. In general, you never want to define this!
 */
#undef	SECURE

#define SEARCHMAP "permissions"

#define LX_SP	0x00
#define LX_EOF	0x01
#define LX_C	0x02
#define LX_X	0x04
#define LX_SC	0x08
#define LX_S1	0x10
static char lookup[] = {
	0x01, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 	/* 00-07 */
	0x04, 0x00, 0x01, 0x04, 0x04, 0x01, 0x04, 0x04, 	/* 08-0f */
	0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 	/* 10-17 */
	0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 	/* 18-1f */
	0x00, 0x02, 0x04, 0x04, 0x02, 0x04, 0x02, 0x04, 	/* 20-27 */
	0x18, 0x18, 0x18, 0x02, 0x02, 0x0a, 0x18, 0x18, 	/* 28-2f */
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 	/* 30-37 */
	0x18, 0x18, 0x02, 0x04, 0x04, 0x04, 0x04, 0x18, 	/* 38-3f */
	0x02, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 	/* 40-47 */
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 	/* 48-4f */
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 	/* 50-57 */
	0x18, 0x18, 0x18, 0x18, 0x04, 0x18, 0x18, 0x18, 	/* 58-5f */
	0x04, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 	/* 60-67 */
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 	/* 68-6f */
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 	/* 70-77 */
	0x18, 0x18, 0x18, 0x04, 0x02, 0x04, 0x04, 0x04, 	/* 78-7f */
};

#define YYSTYPE caddr_t
extern YYSTYPE yylval;

static char *grplst[NGROUPS], **grplist;
char domain[MAXHOSTNAMELEN], key[MAXHOSTNAMELEN];
static char *usern, *ttyn, *line;
static int ttymatch, grpusrmatch;

static int allow, err_allow;
static char *stack[20];
static int stackp;

static
yyerror() {
	syslog(LOG_CRIT,
		"permissions: [key %s] parsing error\n", key);
	err_allow = 1;
}

/*
 * noisy, specifies to be noisy on failure or not
 */
static char *
getypent(s, noisy)
char *s;
{
	char *entry;
	int entrylen, err;

	err = yp_match(domain, SEARCHMAP, s, strlen(s), &entry, &entrylen);
	if(err) {
		if(noisy) syslog(LOG_CRIT,
			"yperr: [key %s] %s\n", s, yperr_string(err));
		return NULL;
	}
	return entry;
}

/* usr:  user name. Absolutely has to be the login name.
 * tty:  tty name. if NULL, any tty will match.
 * grps: NULL terminated array of group names. If grps==NULL, perms
 *	 will fetch the groups using getpwent() and getgrent(). These
 *	 have static data -- beware.
 * lkey: base lookup key. If NULL, hostname will be used.
 *
 * return values need cleaning up.
 *  1 - allowed
 *  0 - not allowed
 * -1 - error
 */
int
permcheck(usr, tty, grps, lkey)
char *usr, *tty, **grps, *lkey;
{
	struct passwd *pwd;
	struct group *grp;
	int ngrps = 0;

	if(!usr)
		return -1;
	if(!grps) {
		grps = grplst;
#ifdef DEBUG
		printf("user: %s\n", usr);
#endif
		setpwent();
		if( !(pwd=getpwnam(usr)))
			return -1;
#ifdef DEBUG
		printf("user: (%s)\n", pwd->pw_name);
#endif

		setgrent();
		if( !(grp=getgrgid(pwd->pw_gid)) )
			return -1;
		*grps++ = strdup(grp->gr_name);
#ifdef DEBUG
		printf("group: %s\n", grp->gr_name);
#endif
		ngrps++;
		while( grp=getgrent() ) {
			if(pwd->pw_gid == grp->gr_gid)
				continue;
			while(*grp->gr_mem) {
				if(!strcmp(*grp->gr_mem, usr)) {
					if(++ngrps>NGROUPS)
						break;
					*grps++ = strdup(grp->gr_name);
#ifdef DEBUG
					printf("group: %s\n", grp->gr_name);
#endif
				}
				grp->gr_mem++;
			}
		}
		endgrent();
		endpwent();
		*grps = NULL;
		grps = grplst;
	}
	if(lkey)
		strcpy(key, lkey);
	else
		gethostname(key, sizeof key);

	getdomainname(domain, sizeof domain);
	if( !(line=getypent(key, 0)) ) {
		line = getypent("*", 1);
	}
	allow = 0;
	err_allow = !line;

	if(line) {
		usern = usr;
		ttyn = tty;
		grplist = grps;
		ttymatch = grpusrmatch = -1;
		yyparse();
	}

#ifdef SECURE
	return allow;
#else
	return allow | err_allow;
#endif
}

static
newmap(s)
char *s;
{
#ifdef DEBUG
	printf("include %s, pushing %s", s, line);
#endif
	stack[stackp++] = line;
	if(stackp==sizeof(stack)/sizeof(char *)) {
		syslog(LOG_CRIT,
			"permissions: stack overflow on [key %s]\n", s);
		line = stack[--stackp];
		err_allow = 1;
		return;
	}
	line = getypent(s,1);
	if(!line) {
		syslog(LOG_CRIT,
			"permissions: [key %s] not found\n", s);
		line = stack[--stackp];
		err_allow = 1;
		return;
	}
#ifdef DEBUG
	printf("result %s", line);
#endif
}

static
yylex()
{
	static pos = 0;
	static char savebuf[1024], *p;
	char c;

restartlex:
	p = savebuf;
	while(1)
		switch( lookup[(c= *line++)] ) {
		case LX_SP:
			break;
		case LX_EOF:
			if(!stackp)
				return -1;
			line = stack[--stackp];
#ifdef DEBUG
			printf("popping %s", line);
#endif
			goto restartlex;
		case LX_C:
		case LX_C|LX_SC:
#ifdef DEBUG
			printf("lex: char '%c'\n", c);
#endif
			return c;
		case LX_X:
			syslog(LOG_CRIT,
				"permissions: [key %s] illegal char 0%3o\n",
				key, c);
			err_allow = 1;
			return -1;
		case LX_S1:
		case LX_S1|LX_SC:
			do {
				*p++ = c;
				c = *line++;
			} while( lookup[c]&LX_SC );
			*p = '\0';
			line--;
			yylval = savebuf;
#ifdef DEBUG
			printf("lex: symbol %s\n", savebuf);
#endif
			return SYMBOL;
		}
}

static void
grpcheck(s, add)
char *s;
{
	FILE *fin;
	char **grp, *p, buf[80];
	int i;

	switch(*s) {
	case '(':
		if( strchr(s+1, '(')) {
			syslog(LOG_CRIT,
				"permissions: [key %s] can't nest brackets\n", s);
			return;
		}

		p = strchr(s, ')');
		if(p)
			*p = '\0';
		else {
			syslog(LOG_CRIT,
				"permissions: [key %s] include syntax error\n", s);
			return;
		}
		if( strchr(s, ')')) {
			syslog(LOG_CRIT,
				"permissions: [key %s] can't nest brackets\n", s);
			return;
		}

		s++;
		fin = fopen(s, "r");
		if(!fin) {
			syslog(LOG_CRIT,
				"permissions: [key %s] can't open include\n", s);
			return;
		}
		while( fgets(buf, sizeof buf, fin) ) {
			for(p=buf; *p; p++)
				if( *p==' ' || *p=='\n' || *p=='#' || *p=='\t' ) {
					*p = '\0';
					break;
				}
			if(!*buf)
				continue;
#ifdef DEBUG
			printf("file: %s '%s'\n", s, buf);
#endif
			grpcheck(buf, add);
		}
		fclose(fin);
		break;
	case '.':
		grp = grplist;
		while(*grp) {
			if( glob_match(s+1, *grp, 0) )
				grpusrmatch = add;
			grp++;
		}
		break;
	default:
		if( !strcmp(s, usern) )
			grpusrmatch = add;
		break;
	}
}

%}

%token SYMBOL
%start permlist

%%
permlist	: perm
		| permlist '|' perm
		;

perm		: ttylist ':' grouplist {
			if(ttymatch!=-1 && grpusrmatch!=-1)
				allow = ttymatch && grpusrmatch;
			ttymatch = grpusrmatch = -1;
		}
		| '$' SYMBOL { newmap($2); } permlist
		|
		;

ttylist		: tty
		| ttylist ',' tty
		;
grouplist	: group
		| grouplist ',' group
		;

tty		: SYMBOL {
			if(!ttyn)
				ttymatch = 1;
			else if( glob_match($1, ttyn, 0) )
				ttymatch = 1;
		}
		;

group		: '+' SYMBOL	{ grpcheck($2, 1); }
		| '-' SYMBOL	{ grpcheck($2, 0); }
		;
%%
